/*
---

script: MooEditable.js

description: Class for creating a WYSIWYG editor, for contentEditable-capable browsers.

license: MIT-style license

authors:
- Lim Chee Aun
- Radovan Lozej
- Ryan Mitchell
- Olivier Refalo
- T.J. Leahy

requires:
  core/1.2.4:
  - Events
  - Options
  - Element.Event
  - Element.Style
  - Element.Dimensions
  - Selectors

inspiration:
- Code inspired by Stefan's work [Safari Supports Content Editing!](http://www.xs4all.nl/~hhijdra/stefan/ContentEditable.html) from [safari gets contentEditable](http://walkah.net/blog/walkah/safari-gets-contenteditable)
- Main reference from Peter-Paul Koch's [execCommand compatibility](http://www.quirksmode.org/dom/execCommand.html)
- Some ideas and code inspired by [TinyMCE](http://tinymce.moxiecode.com/)
- Some functions inspired by Inviz's [Most tiny wysiwyg you ever seen](http://forum.mootools.net/viewtopic.php?id=746), [mooWyg (Most tiny WYSIWYG 2.0)](http://forum.mootools.net/viewtopic.php?id=5740)
- Some regex from Cameron Adams's [widgEditor](http://widgeditor.googlecode.com/)
- Some code from Juan M Martinez's [jwysiwyg](http://jwysiwyg.googlecode.com/)
- Some reference from MoxieForge's [PunyMCE](http://punymce.googlecode.com/)
- IE support referring Robert Bredlau's [Rich Text Editing](http://www.rbredlau.com/drupal/node/6)
- Tango icons from the [Tango Desktop Project](http://tango.freedesktop.org/)
- Additional Tango icons from Jimmacs' [Tango OpenOffice](http://www.gnome-look.org/content/show.php/Tango+OpenOffice?content=54799)

provides: [MooEditable, MooEditable.Selection, MooEditable.UI, MooEditable.Actions]

...
*/
(function(){var a=/^(H[1-6]|HR|P|DIV|ADDRESS|PRE|FORM|TABLE|LI|OL|UL|TD|CAPTION|BLOCKQUOTE|CENTER|DL|DT|DD)$/i;var b=/^(https?|ftp|rmtp|mms):\/\/(([A-Z0-9][A-Z0-9_-]*)(\.[A-Z0-9][A-Z0-9_-]*)+)(:(\d+))?\/?/i;this.MooEditable=new Class({Implements:[Events,Options],options:{toolbar:true,cleanup:true,paragraphise:true,xhtml:true,semantics:true,actions:"bold italic underline strikethrough | insertunorderedlist insertorderedlist indent outdent | undo redo | createlink unlink | urlimage | toggleview",handleSubmit:true,handleLabel:true,baseCSS:"html{ height: 100%; cursor: text; } body{ font-family: sans-serif; }",extraCSS:"",externalCSS:"",html:'<!DOCTYPE html><html><head><meta http-equiv="Content-Type" content="text/html; charset=UTF-8"><style>{BASECSS} {EXTRACSS}</style>{EXTERNALCSS}</head><body>{CONTENT}</body></html>',rootElement:"p"},initialize:function(d,c){this.setOptions(c);this.textarea=document.id(d);this.textarea.store("MooEditable",this);this.actions=this.options.actions.clean().split(" ");this.keys={};this.dialogs={};this.actions.each(function(g){var e=MooEditable.Actions[g];if(!e){return}if(e.options){var f=e.options.shortcut;if(f){this.keys[f]=g}}if(e.dialogs){$each(e.dialogs,function(j,h){j=j.attempt(this);j.name=g+":"+h;if($type(this.dialogs[g])!="object"){this.dialogs[g]={}}this.dialogs[g][h]=j},this)}if(e.events){$each(e.events,function(h,j){this.addEvent(j,h)},this)}}.bind(this));this.render()},toElement:function(){return this.textarea},render:function(){var c=this;var d=this.textarea.getSize();this.container=new Element("div",{id:(this.textarea.id)?this.textarea.id+"-mooeditable-container":null,"class":"mooeditable-container",styles:{width:d.x}});this.textarea.addClass("mooeditable-textarea").setStyle("height",d.y);this.iframe=new IFrame({"class":"mooeditable-iframe",frameBorder:0,src:'javascript:""',styles:{height:d.y}});this.toolbar=new MooEditable.UI.Toolbar({onItemAction:function(){var e=$splat(arguments);var f=e[0];c.action(f.name,e)}});this.attach.delay(1,this);if(this.options.handleLabel&&this.textarea.id){$$('label[for="'+this.textarea.id+'"]').addEvent("click",function(f){if(c.mode!="iframe"){return}f.preventDefault();c.focus()})}if(this.options.handleSubmit){this.form=this.textarea.getParent("form");if(!this.form){return}this.form.addEvent("submit",function(){if(c.mode=="iframe"){c.saveContent()}})}this.fireEvent("render",this)},attach:function(){var c=this;this.mode="iframe";this.editorDisabled=false;this.container.wraps(this.textarea);this.textarea.setStyle("display","none");this.iframe.setStyle("display","").inject(this.textarea,"before");$each(this.dialogs,function(g,f){$each(g,function(j){document.id(j).inject(c.iframe,"before");var h;j.addEvents({open:function(){h=c.selection.getRange();c.editorDisabled=true;c.toolbar.disable(f);c.fireEvent("dialogOpen",this)},close:function(){c.toolbar.enable();c.editorDisabled=false;c.focus();if(h){c.selection.setRange(h)}c.fireEvent("dialogClose",this)}})})});this.win=this.iframe.contentWindow;this.doc=this.win.document;var e=this.options.html.substitute({BASECSS:this.options.baseCSS,EXTRACSS:this.options.extraCSS,EXTERNALCSS:(this.options.externalCSS)?'<link rel="stylesheet" href="'+this.options.externalCSS+'">':"",CONTENT:this.cleanup(this.textarea.get("value"))});this.doc.open();this.doc.write(e);this.doc.close();(Browser.Engine.trident)?this.doc.body.contentEditable=true:this.doc.designMode="On";if(!this.win.$family){new Window(this.win)}if(!this.doc.$family){new Document(this.doc)}document.id(this.doc.body);this.doc.addEvents({mouseup:this.editorMouseUp.bind(this),mousedown:this.editorMouseDown.bind(this),mouseover:this.editorMouseOver.bind(this),mouseout:this.editorMouseOut.bind(this),mouseenter:this.editorMouseEnter.bind(this),mouseleave:this.editorMouseLeave.bind(this),contextmenu:this.editorContextMenu.bind(this),click:this.editorClick.bind(this),dbllick:this.editorDoubleClick.bind(this),keypress:this.editorKeyPress.bind(this),keyup:this.editorKeyUp.bind(this),keydown:this.editorKeyDown.bind(this),focus:this.editorFocus.bind(this),blur:this.editorBlur.bind(this)});this.win.addEvents({focus:this.editorFocus.bind(this),blur:this.editorBlur.bind(this)});["cut","copy","paste"].each(function(f){c.doc.body.addListener(f,c["editor"+f.capitalize()].bind(c))});this.textarea.addEvent("keypress",this.textarea.retrieve("mooeditable:textareaKeyListener",this.keyListener.bind(this)));if(Browser.Engine.gecko&&Browser.Engine.version==18){this.doc.addEvent("focus",function(){c.win.fireEvent("focus").focus()})}if(!(/trident|presto/i).test(Browser.Engine.name)){var d=function(){c.execute("styleWithCSS",false,false);c.doc.removeEvent("focus",d)};this.win.addEvent("focus",d)}if(this.options.toolbar){document.id(this.toolbar).inject(this.container,"top");this.toolbar.render(this.actions)}this.selection=new MooEditable.Selection(this.win);this.oldContent=this.getContent();this.fireEvent("attach",this);return this},detach:function(){this.saveContent();this.textarea.setStyle("display","").removeClass("mooeditable-textarea").inject(this.container,"before");this.textarea.removeEvent("keypress",this.textarea.retrieve("mooeditable:textareaKeyListener"));this.container.dispose();this.fireEvent("detach",this);return this},editorFocus:function(c){this.oldContent="";this.fireEvent("editorFocus",[c,this])},editorBlur:function(c){this.oldContent=this.saveContent().getContent();this.fireEvent("editorBlur",[c,this])},editorMouseUp:function(c){if(this.editorDisabled){c.stop();return}if(this.options.toolbar){this.checkStates()}this.fireEvent("editorMouseUp",[c,this])},editorMouseDown:function(c){if(this.editorDisabled){c.stop();return}this.fireEvent("editorMouseDown",[c,this])},editorMouseOver:function(c){if(this.editorDisabled){c.stop();return}this.fireEvent("editorMouseOver",[c,this])},editorMouseOut:function(c){if(this.editorDisabled){c.stop();return}this.fireEvent("editorMouseOut",[c,this])},editorMouseEnter:function(c){if(this.editorDisabled){c.stop();return}if(this.oldContent&&this.getContent()!=this.oldContent){this.focus();this.fireEvent("editorPaste",[c,this])}this.fireEvent("editorMouseEnter",[c,this])},editorMouseLeave:function(c){if(this.editorDisabled){c.stop();return}this.fireEvent("editorMouseLeave",[c,this])},editorContextMenu:function(c){if(this.editorDisabled){c.stop();return}this.fireEvent("editorContextMenu",[c,this])},editorClick:function(d){if(Browser.Engine.webkit){var c=d.target;if(c.get("tag")=="img"){this.selection.selectNode(c)}}this.fireEvent("editorClick",[d,this])},editorDoubleClick:function(c){this.fireEvent("editorDoubleClick",[c,this])},editorKeyPress:function(c){if(this.editorDisabled){c.stop();return}this.keyListener(c);this.fireEvent("editorKeyPress",[c,this])},editorKeyUp:function(d){if(this.editorDisabled){d.stop();return}var f=d.code;if(this.options.toolbar&&(/^enter|left|up|right|down|delete|backspace$/i.test(d.key)||(f>=33&&f<=36)||f==45||d.meta||d.control)){if(Browser.Engines.trident4){$clear(this.checkStatesDelay);this.checkStatesDelay=this.checkStates.delay(500,this)}else{this.checkStates()}}this.fireEvent("editorKeyUp",[d,this])},editorKeyDown:function(j){if(this.editorDisabled){j.stop();return}if(j.key=="enter"){if(this.options.paragraphise){if(j.shift&&Browser.Engine.webkit){var o=this.selection;var c=o.getRange();var n=this.doc.createElement("br");c.insertNode(n);c.setStartAfter(n);c.setEndAfter(n);o.setRange(c);if(o.getSelection().focusNode==n.previousSibling){var m=this.doc.createTextNode("\u00a0");var d=n.parentNode;var l=n.nextSibling;(l)?d.insertBefore(m,l):d.appendChild(m);o.selectNode(m);o.collapse(1)}this.win.scrollTo(0,Element.getOffsets(o.getRange().startContainer).y);j.preventDefault()}else{if(Browser.Engine.gecko||Browser.Engine.webkit){var g=this.selection.getNode();var f=g.getParents().include(g).some(function(e){return e.nodeName.test(a)});if(!f){this.execute("insertparagraph")}}}}else{if(Browser.Engine.trident){var c=this.selection.getRange();var g=this.selection.getNode();if(c&&g.get("tag")!="li"){this.selection.insertContent("<br>");this.selection.collapse(false)}j.preventDefault()}}}if(Browser.Engine.presto){var h=j.control||j.meta;if(h&&j.key=="x"){this.fireEvent("editorCut",[j,this])}else{if(h&&j.key=="c"){this.fireEvent("editorCopy",[j,this])}else{if((h&&j.key=="v")||(j.shift&&j.code==45)){this.fireEvent("editorPaste",[j,this])}}}}this.fireEvent("editorKeyDown",[j,this])},editorCut:function(c){if(this.editorDisabled){c.stop();return}this.fireEvent("editorCut",[c,this])},editorCopy:function(c){if(this.editorDisabled){c.stop();return}this.fireEvent("editorCopy",[c,this])},editorPaste:function(c){if(this.editorDisabled){c.stop();return}this.fireEvent("editorPaste",[c,this])},keyListener:function(f){var c=(Browser.Platform.mac)?f.meta:f.control;if(!c||!this.keys[f.key]){return}f.preventDefault();var d=this.toolbar.getItem(this.keys[f.key]);d.action(f)},focus:function(){(this.mode=="iframe"?this.win:this.textarea).focus();this.fireEvent("focus",this);return this},action:function(e,c){var d=MooEditable.Actions[e];if(d.command&&$type(d.command)=="function"){d.command.run(c,this)}else{this.focus();this.execute(e,false,c);if(this.mode=="iframe"){this.checkStates()}}},execute:function(e,d,c){if(this.busy){return}this.busy=true;this.doc.execCommand(e,d,c);this.saveContent();this.busy=false;return false},toggleView:function(){this.fireEvent("beforeToggleView",this);if(this.mode=="textarea"){this.mode="iframe";this.iframe.setStyle("display","");this.setContent(this.textarea.value);this.textarea.setStyle("display","none")}else{this.saveContent();this.mode="textarea";this.textarea.setStyle("display","");this.iframe.setStyle("display","none")}this.fireEvent("toggleView",this);this.focus.delay(10,this);return this},getContent:function(){return this.cleanup(this.doc.body.get("html"))},setContent:function(c){this.doc.body.set("html",this.ensureRootElement(c));return this},saveContent:function(){if(this.mode=="iframe"){this.textarea.set("value",this.ensureRootElement(this.getContent()))}return this},ensureRootElement:function(h){if(this.options.rootElement){var g=new Element("div",{html:h.trim()});var e=-1;var j=false;var l="";var f=g.childNodes.length;for(i=0;i<f;i++){var c=g.childNodes[i];var m=c.nodeName;if(!m.test(a)&&m!=="#comment"){if(m==="#text"){if(c.nodeValue.trim()){if(e<0){e=i}l+=c.nodeValue}}else{if(e<0){e=i}l+=new Element("div").adopt($(c).clone()).get("html")}}else{j=true}if(i==(f-1)){j=true}if(e>=0&&j){var d=new Element(this.options.rootElement,{html:l});g.replaceChild(d,g.childNodes[e]);for(k=e+1;k<i;k++){g.removeChild(g.childNodes[k]);f--;i--;k--}e=-1;j=false;l=""}}h=g.get("html").replace(/\n\n/g,"")}return h},checkStates:function(){var c=this.selection.getNode();if(!c){return}if($type(c)!="element"){return}this.actions.each(function(g){var l=this.toolbar.getItem(g);if(!l){return}l.deactivate();var m=MooEditable.Actions[g]["states"];if(!m){return}if($type(m)=="function"){m.attempt([document.id(c),l],this);return}try{if(this.doc.queryCommandState(g)){l.activate();return}}catch(j){}if(m.tags){var f=c;do{var o=f.tagName.toLowerCase();if(m.tags.contains(o)){l.activate(o);break}}while((f=Element.getParent(f))!=null)}if(m.css){var f=c;do{var n=false;for(var d in m.css){var h=m.css[d];if(Element.getStyle(f,d).contains(h)){l.activate(h);n=true}}if(n||f.tagName.test(a)){break}}while((f=Element.getParent(f))!=null)}}.bind(this))},cleanup:function(d){if(!this.options.cleanup){return d.trim()}do{var c=d;d=d.replace(/<br class\="webkit-block-placeholder">/gi,"<br />");d=d.replace(/<span class="Apple-style-span">(.*)<\/span>/gi,"$1");d=d.replace(/ class="Apple-style-span"/gi,"");d=d.replace(/<span style="">/gi,"");d=d.replace(/<p>\s*<br ?\/?>\s*<\/p>/gi,"<p>\u00a0</p>");d=d.replace(/<p>(&nbsp;|\s)*<\/p>/gi,"<p>\u00a0</p>");if(!this.options.semantics){d=d.replace(/\s*<br ?\/?>\s*<\/p>/gi,"</p>")}if(this.options.xhtml){d=d.replace(/<br>/gi,"<br />")}if(this.options.semantics){if(Browser.Engine.trident){d=d.replace(/<li>\s*<div>(.+?)<\/div><\/li>/g,"<li>$1</li>")}if(Browser.Engine.webkit){d=d.replace(/^([\w\s]+.*?)<div>/i,"<p>$1</p><div>");d=d.replace(/<div>(.+?)<\/div>/ig,"<p>$1</p>")}if(["gecko","presto","webkit"].contains(Browser.Engine.name)){d=d.replace(/<p>[\s\n]*(<(?:ul|ol)>.*?<\/(?:ul|ol)>)(.*?)<\/p>/ig,"$1<p>$2</p>");d=d.replace(/<\/(ol|ul)>\s*(?!<(?:p|ol|ul|img).*?>)((?:<[^>]*>)?\w.*)$/g,"</$1><p>$2</p>")}d=d.replace(/<br[^>]*><\/p>/g,"</p>");d=d.replace(/<p>\s*(<img[^>]+>)\s*<\/p>/ig,"$1\n");d=d.replace(/<p([^>]*)>(.*?)<\/p>(?!\n)/g,"<p$1>$2</p>\n");d=d.replace(/<\/(ul|ol|p)>(?!\n)/g,"</$1>\n");d=d.replace(/><li>/g,">\n\t<li>");d=d.replace(/([^\n])<\/(ol|ul)>/g,"$1\n</$2>");d=d.replace(/([^\n])<img/ig,"$1\n<img");d=d.replace(/^\s*$/g,"")}d=d.replace(/<br ?\/?>$/gi,"");d=d.replace(/^<br ?\/?>/gi,"");d=d.replace(/><br ?\/?>/gi,">");d=d.replace(/<br ?\/?>\s*<\/(h1|h2|h3|h4|h5|h6|li|p)/gi,"</$1");d=d.replace(/<span style="font-weight: bold;">(.*)<\/span>/gi,"<strong>$1</strong>");d=d.replace(/<span style="font-style: italic;">(.*)<\/span>/gi,"<em>$1</em>");d=d.replace(/<b\b[^>]*>(.*?)<\/b[^>]*>/gi,"<strong>$1</strong>");d=d.replace(/<i\b[^>]*>(.*?)<\/i[^>]*>/gi,"<em>$1</em>");d=d.replace(/<u\b[^>]*>(.*?)<\/u[^>]*>/gi,'<span style="text-decoration: underline;">$1</span>');d=d.replace(/<strong><span style="font-weight: normal;">(.*)<\/span><\/strong>/gi,"$1");d=d.replace(/<em><span style="font-weight: normal;">(.*)<\/span><\/em>/gi,"$1");d=d.replace(/<span style="text-decoration: underline;"><span style="font-weight: normal;">(.*)<\/span><\/span>/gi,"$1");d=d.replace(/<strong style="font-weight: normal;">(.*)<\/strong>/gi,"$1");d=d.replace(/<em style="font-weight: normal;">(.*)<\/em>/gi,"$1");d=d.replace(/<[^> ]*/g,function(e){return e.toLowerCase()});d=d.replace(/<[^>]*>/g,function(e){e=e.replace(/ [^=]+=/g,function(f){return f.toLowerCase()});return e});d=d.replace(/<[^>]*>/g,function(e){e=e.replace(/( [^=]+=)([^"][^ >]*)/g,'$1"$2"');return e});if(this.options.xhtml){d=d.replace(/<img([^>]+)(\s*[^\/])>(<\/img>)*/gi,"<img$1$2 />")}d=d.replace(/<p>(?:\s*)<p>/g,"<p>");d=d.replace(/<\/p>\s*<\/p>/g,"</p>");d=d.replace(/<pre[^>]*>.*?<\/pre>/gi,function(e){return e.replace(/<br ?\/?>/gi,"\n")});d=d.trim()}while(d!=c);return d}});MooEditable.Selection=new Class({initialize:function(c){this.win=c},getSelection:function(){this.win.focus();return(this.win.getSelection)?this.win.getSelection():this.win.document.selection},getRange:function(){var c=this.getSelection();if(!c){return null}try{return c.rangeCount>0?c.getRangeAt(0):(c.createRange?c.createRange():null)}catch(d){return this.doc.body.createTextRange()}},setRange:function(c){if(c.select){$try(function(){c.select()})}else{var d=this.getSelection();if(d.addRange){d.removeAllRanges();d.addRange(c)}}},selectNode:function(e,f){var d=this.getRange();var c=this.getSelection();if(d.moveToElementText){$try(function(){d.moveToElementText(e);d.select()})}else{if(c.addRange){f?d.selectNodeContents(e):d.selectNode(e);c.removeAllRanges();c.addRange(d)}else{c.setBaseAndExtent(e,0,e,1)}}return e},isCollapsed:function(){var c=this.getRange();if(c.item){return false}return c.boundingWidth==0||this.getSelection().isCollapsed},collapse:function(c){var e=this.getRange();var d=this.getSelection();if(e.select){e.collapse(c);e.select()}else{c?d.collapseToStart():d.collapseToEnd()}},getContent:function(){var e=this.getRange();var c=new Element("body");if(this.isCollapsed()){return""}if(e.cloneContents){c.appendChild(e.cloneContents())}else{if($defined(e.item)||$defined(e.htmlText)){c.set("html",e.item?e.item(0).outerHTML:e.htmlText)}else{c.set("html",e.toString())}}var d=c.get("html");return d},getText:function(){var d=this.getRange();var c=this.getSelection();return this.isCollapsed()?"":d.text||(c.toString?c.toString():"")},getNode:function(){var d=this.getRange();if(!Browser.Engine.trident){var c=null;if(d){c=d.commonAncestorContainer;if(!d.collapsed){if(d.startContainer==d.endContainer){if(d.startOffset-d.endOffset<2){if(d.startContainer.hasChildNodes()){c=d.startContainer.childNodes[d.startOffset]}}}}while($type(c)!="element"){c=c.parentNode}}return document.id(c)}return document.id(d.item?d.item(0):d.parentElement())},insertContent:function(d){if(Browser.Engine.trident){var c=this.getRange();c.pasteHTML(d);c.collapse(false);c.select()}else{this.win.document.execCommand("insertHTML",false,d)}}});MooEditable.UI={};MooEditable.UI.Toolbar=new Class({Implements:[Events,Options],options:{"class":""},initialize:function(c){this.setOptions(c);this.el=new Element("div",{"class":"mooeditable-ui-toolbar "+this.options["class"]});this.items={};this.content=null},toElement:function(){return this.el},render:function(c){if(this.content){this.el.adopt(this.content)}else{this.content=c.map(function(d){return(d=="|")?this.addSeparator():this.addItem(d)}.bind(this))}return this},addItem:function(h){var d=this;var c=MooEditable.Actions[h];if(!c){return}var f=c.type||"button";var e=c.options||{};var g=new MooEditable.UI[f.camelCase().capitalize()]($extend(e,{name:h,"class":h+"-item toolbar-item",title:c.title,onAction:d.itemAction.bind(d)}));this.items[h]=g;document.id(g).inject(this.el);return g},getItem:function(c){return this.items[c]},addSeparator:function(){return new Element("span",{"class":"toolbar-separator"}).inject(this.el)},itemAction:function(){this.fireEvent("itemAction",arguments)},disable:function(c){$each(this.items,function(d){(d.name==c)?d.activate():d.deactivate().disable()});return this},enable:function(){$each(this.items,function(c){c.enable()});return this},show:function(){this.el.setStyle("display","");return this},hide:function(){this.el.setStyle("display","none");return this}});MooEditable.UI.Button=new Class({Implements:[Events,Options],options:{title:"",name:"",text:"Button","class":"",shortcut:"",mode:"icon"},initialize:function(c){this.setOptions(c);this.name=this.options.name;this.render()},toElement:function(){return this.el},render:function(){var d=this;var e=(Browser.Platform.mac)?"Cmd":"Ctrl";var c=(this.options.shortcut)?" ( "+e+"+"+this.options.shortcut.toUpperCase()+" )":"";var g=this.options.title||name;var f=g+c;this.el=new Element("button",{"class":"mooeditable-ui-button "+d.options["class"],title:f,html:'<span class="button-icon"></span><span class="button-text">'+g+"</span>",events:{click:d.click.bind(d),mousedown:function(h){h.preventDefault()}}});if(this.options.mode!="icon"){this.el.addClass("mooeditable-ui-button-"+this.options.mode)}this.active=false;this.disabled=false;if(Browser.Engine.trident){this.el.addEvents({mouseenter:function(h){this.addClass("hover")},mouseleave:function(h){this.removeClass("hover")}})}return this},click:function(c){c.preventDefault();if(this.disabled){return}this.action(c)},action:function(){this.fireEvent("action",[this].concat($A(arguments)))},enable:function(){if(this.active){this.el.removeClass("onActive")}if(!this.disabled){return}this.disabled=false;this.el.removeClass("disabled").set({disabled:false,opacity:1});return this},disable:function(){if(this.disabled){return}this.disabled=true;this.el.addClass("disabled").set({disabled:true,opacity:0.4});return this},activate:function(){if(this.disabled){return}this.active=true;this.el.addClass("onActive");return this},deactivate:function(){this.active=false;this.el.removeClass("onActive");return this}});MooEditable.UI.Dialog=new Class({Implements:[Events,Options],options:{"class":"",contentClass:""},initialize:function(e,d){this.setOptions(d);this.html=e;var c=this;this.el=new Element("div",{"class":"mooeditable-ui-dialog "+c.options["class"],html:'<div class="dialog-content '+c.options.contentClass+'">'+e+"</div>",styles:{display:"none"},events:{click:c.click.bind(c)}})},toElement:function(){return this.el},click:function(){this.fireEvent("click",arguments);return this},open:function(){this.el.setStyle("display","");this.fireEvent("open",this);return this},close:function(){this.el.setStyle("display","none");this.fireEvent("close",this);return this}});MooEditable.UI.AlertDialog=function(d){if(!d){return}var c=d+' <button class="dialog-ok-button">OK</button>';return new MooEditable.UI.Dialog(c,{"class":"mooeditable-alert-dialog",onOpen:function(){var e=this.el.getElement(".dialog-ok-button");(function(){e.focus()}).delay(10)},onClick:function(f){f.preventDefault();if(f.target.tagName.toLowerCase()!="button"){return}if(document.id(f.target).hasClass("dialog-ok-button")){this.close()}}})};MooEditable.UI.PromptDialog=function(e,f,d){if(!e){return}var c='<label class="dialog-label">'+e+' <input type="text" class="text dialog-input" value="'+f+'"></label> <button class="dialog-button dialog-ok-button">OK</button><button class="dialog-button dialog-cancel-button">Cancel</button>';return new MooEditable.UI.Dialog(c,{"class":"mooeditable-prompt-dialog",onOpen:function(){var g=this.el.getElement(".dialog-input");(function(){g.focus();g.select()}).delay(10)},onClick:function(l){l.preventDefault();if(l.target.tagName.toLowerCase()!="button"){return}var h=document.id(l.target);var g=this.el.getElement(".dialog-input");if(h.hasClass("dialog-cancel-button")){g.set("value",f);this.close()}else{if(h.hasClass("dialog-ok-button")){var j=g.get("value");g.set("value",f);this.close();if(d){d.attempt(j,this)}}}}})};MooEditable.Actions=new Hash({bold:{title:"Bold",options:{shortcut:"b"},states:{tags:["b","strong"],css:{"font-weight":"bold"}},events:{beforeToggleView:function(){if(Browser.Engine.gecko){var c=this.textarea.get("value").replace(/<strong([^>]*)>/gi,"<b$1>").replace(/<\/strong>/gi,"</b>");this.textarea.set("value",c)}},attach:function(){if(Browser.Engine.gecko){var c=this.textarea.get("value").replace(/<strong([^>]*)>/gi,"<b$1>").replace(/<\/strong>/gi,"</b>");this.textarea.set("value",c);this.setContent(c)}}}},italic:{title:"Italic",options:{shortcut:"i"},states:{tags:["i","em"],css:{"font-style":"italic"}},events:{beforeToggleView:function(){if(Browser.Engine.gecko){var c=this.textarea.get("value").replace(/<embed([^>]*)>/gi,"<tmpembed$1>").replace(/<em([^>]*)>/gi,"<i$1>").replace(/<tmpembed([^>]*)>/gi,"<embed$1>").replace(/<\/em>/gi,"</i>");this.textarea.set("value",c)}},attach:function(){if(Browser.Engine.gecko){var c=this.textarea.get("value").replace(/<embed([^>]*)>/gi,"<tmpembed$1>").replace(/<em([^>]*)>/gi,"<i$1>").replace(/<tmpembed([^>]*)>/gi,"<embed$1>").replace(/<\/em>/gi,"</i>");this.textarea.set("value",c);this.setContent(c)}}}},underline:{title:"Underline",options:{shortcut:"u"},states:{tags:["u"],css:{"text-decoration":"underline"}}},strikethrough:{title:"Strikethrough",options:{shortcut:"s"},states:{tags:["s","strike"],css:{"text-decoration":"line-through"}}},insertunorderedlist:{title:"Unordered List",states:{tags:["ul"]}},insertorderedlist:{title:"Ordered List",states:{tags:["ol"]}},indent:{title:"Indent",states:{tags:["blockquote"]}},outdent:{title:"Outdent"},undo:{title:"Undo",options:{shortcut:"z"}},redo:{title:"Redo",options:{shortcut:"y"}},unlink:{title:"Remove Hyperlink"},createlink:{title:"Add Hyperlink",options:{shortcut:"l"},states:{tags:["a"]},dialogs:{alert:MooEditable.UI.AlertDialog.pass("Please select the text you wish to hyperlink."),prompt:function(c){return MooEditable.UI.PromptDialog("Enter URL","http://",function(d){c.execute("createlink",false,d.trim())})}},command:function(){if(this.selection.isCollapsed()){this.dialogs.createlink.alert.open()}else{var d=this.selection.getText();var c=this.dialogs.createlink.prompt;if(b.test(d)){c.el.getElement(".dialog-input").set("value",d)}c.open()}}},urlimage:{title:"Add Image",options:{shortcut:"m"},dialogs:{prompt:function(c){return MooEditable.UI.PromptDialog("Enter image URL","http://",function(d){c.execute("insertimage",false,d.trim())})}},command:function(){this.dialogs.urlimage.prompt.open()}},toggleview:{title:"Toggle View",command:function(){(this.mode=="textarea")?this.toolbar.enable():this.toolbar.disable("toggleview");this.toggleView()}}});MooEditable.Actions.Settings={};Element.Properties.mooeditable={set:function(c){return this.eliminate("mooeditable").store("mooeditable:options",c)},get:function(c){if(c||!this.retrieve("mooeditable")){if(c||!this.retrieve("mooeditable:options")){this.set("mooeditable",c)}this.store("mooeditable",new MooEditable(this,this.retrieve("mooeditable:options")))}return this.retrieve("mooeditable")}};Element.implement({mooEditable:function(c){return this.get("mooeditable",c)}})})();MooEditable.Actions.Settings.charmap={chars:[["&nbsp;","&#160;"],["&amp;","&#38;"],["&quot;","&#34;"],["&cent;","&#162;"],["&euro;","&#8364;"],["&pound;","&#163;"],["&yen;","&#165;"],["&copy;","&#169;"],["&reg;","&#174;"],["&trade;","&#8482;"],["&permil;","&#8240;"],["&micro;","&#181;"],["&middot;","&#183;"],["&bull;","&#8226;"],["&hellip;","&#8230;"],["&prime;","&#8242;"],["&Prime;","&#8243;"],["&sect;","&#167;"],["&para;","&#182;"],["&szlig;","&#223;"],["&lsaquo;","&#8249;"],["&rsaquo;","&#8250;"],["&laquo;","&#171;"],["&raquo;","&#187;"],["&lsquo;","&#8216;"],["&rsquo;","&#8217;"],["&ldquo;","&#8220;"],["&rdquo;","&#8221;"],["&sbquo;","&#8218;"],["&bdquo;","&#8222;"],["&lt;","&#60;"],["&gt;","&#62;"],["&le;","&#8804;"],["&ge;","&#8805;"],["&ndash;","&#8211;"],["&mdash;","&#8212;"],["&macr;","&#175;"],["&oline;","&#8254;"],["&curren;","&#164;"],["&brvbar;","&#166;"],["&uml;","&#168;"],["&iexcl;","&#161;"],["&iquest;","&#191;"],["&circ;","&#710;"],["&tilde;","&#732;"],["&deg;","&#176;"],["&minus;","&#8722;"],["&plusmn;","&#177;"],["&divide;","&#247;"],["&frasl;","&#8260;"],["&times;","&#215;"],["&sup1;","&#185;"],["&sup2;","&#178;"],["&sup3;","&#179;"],["&frac14;","&#188;"],["&frac12;","&#189;"],["&frac34;","&#190;"],["&fnof;","&#402;"],["&int;","&#8747;"],["&sum;","&#8721;"],["&infin;","&#8734;"],["&radic;","&#8730;"],["&sim;","&#8764;"],["&cong;","&#8773;"],["&asymp;","&#8776;"],["&ne;","&#8800;"],["&equiv;","&#8801;"],["&isin;","&#8712;"],["&notin;","&#8713;"],["&ni;","&#8715;"],["&prod;","&#8719;"],["&and;","&#8743;"],["&or;","&#8744;"],["&not;","&#172;"],["&cap;","&#8745;"],["&cup;","&#8746;"],["&part;","&#8706;"],["&forall;","&#8704;"],["&exist;","&#8707;"],["&empty;","&#8709;"],["&nabla;","&#8711;"],["&lowast;","&#8727;"],["&prop;","&#8733;"],["&ang;","&#8736;"],["&acute;","&#180;"],["&cedil;","&#184;"],["&ordf;","&#170;"],["&ordm;","&#186;"],["&dagger;","&#8224;"],["&Dagger;","&#8225;"],["&Agrave;","&#192;"],["&Aacute;","&#193;"],["&Acirc;","&#194;"],["&Atilde;","&#195;"],["&Auml;","&#196;"],["&Aring;","&#197;"],["&AElig;","&#198;"],["&Ccedil;","&#199;"],["&Egrave;","&#200;"],["&Eacute;","&#201;"],["&Ecirc;","&#202;"],["&Euml;","&#203;"],["&Igrave;","&#204;"],["&Iacute;","&#205;"],["&Icirc;","&#206;"],["&Iuml;","&#207;"],["&ETH;","&#208;"],["&Ntilde;","&#209;"],["&Ograve;","&#210;"],["&Oacute;","&#211;"],["&Ocirc;","&#212;"],["&Otilde;","&#213;"],["&Ouml;","&#214;"],["&Oslash;","&#216;"],["&OElig;","&#338;"],["&Scaron;","&#352;"],["&Ugrave;","&#217;"],["&Uacute;","&#218;"],["&Ucirc;","&#219;"],["&Uuml;","&#220;"],["&Yacute;","&#221;"],["&Yuml;","&#376;"],["&THORN;","&#222;"],["&agrave;","&#224;"],["&aacute;","&#225;"],["&acirc;","&#226;"],["&atilde;","&#227;"],["&auml;","&#228;"],["&aring;","&#229;"],["&aelig;","&#230;"],["&ccedil;","&#231;"],["&egrave;","&#232;"],["&eacute;","&#233;"],["&ecirc;","&#234;"],["&euml;","&#235;"],["&igrave;","&#236;"],["&iacute;","&#237;"],["&icirc;","&#238;"],["&iuml;","&#239;"],["&eth;","&#240;"],["&ntilde;","&#241;"],["&ograve;","&#242;"],["&oacute;","&#243;"],["&ocirc;","&#244;"],["&otilde;","&#245;"],["&ouml;","&#246;"],["&oslash;","&#248;"],["&oelig;","&#339;"],["&scaron;","&#353;"],["&ugrave;","&#249;"],["&uacute;","&#250;"],["&ucirc;","&#251;"],["&uuml;","&#252;"],["&yacute;","&#253;"],["&thorn;","&#254;"],["&yuml;","&#255;"],["&Alpha;","&#913;"],["&Beta;","&#914;"],["&Gamma;","&#915;"],["&Delta;","&#916;"],["&Epsilon;","&#917;"],["&Zeta;","&#918;"],["&Eta;","&#919;"],["&Theta;","&#920;"],["&Iota;","&#921;"],["&Kappa;","&#922;"],["&Lambda;","&#923;"],["&Mu;","&#924;"],["&Nu;","&#925;"],["&Xi;","&#926;"],["&Omicron;","&#927;"],["&Pi;","&#928;"],["&Rho;","&#929;"],["&Sigma;","&#931;"],["&Tau;","&#932;"],["&Upsilon;","&#933;"],["&Phi;","&#934;"],["&Chi;","&#935;"],["&Psi;","&#936;"],["&Omega;","&#937;"],["&alpha;","&#945;"],["&beta;","&#946;"],["&gamma;","&#947;"],["&delta;","&#948;"],["&epsilon;","&#949;"],["&zeta;","&#950;"],["&eta;","&#951;"],["&theta;","&#952;"],["&iota;","&#953;"],["&kappa;","&#954;"],["&lambda;","&#955;"],["&mu;","&#956;"],["&nu;","&#957;"],["&xi;","&#958;"],["&omicron;","&#959;"],["&pi;","&#960;"],["&rho;","&#961;"],["&sigmaf;","&#962;"],["&sigma;","&#963;"],["&tau;","&#964;"],["&upsilon;","&#965;"],["&phi;","&#966;"],["&chi;","&#967;"],["&psi;","&#968;"],["&omega;","&#969;"],["&alefsym;","&#8501;"],["&piv;","&#982;"],["&real;","&#8476;"],["&thetasym;","&#977;"],["&upsih;","&#978;"],["&weierp;","&#8472;"],["&image;","&#8465;"],["&larr;","&#8592;"],["&uarr;","&#8593;"],["&rarr;","&#8594;"],["&darr;","&#8595;"],["&harr;","&#8596;"],["&crarr;","&#8629;"],["&lArr;","&#8656;"],["&uArr;","&#8657;"],["&rArr;","&#8658;"],["&dArr;","&#8659;"],["&hArr;","&#8660;"],["&there4;","&#8756;"],["&sub;","&#8834;"],["&sup;","&#8835;"],["&nsub;","&#8836;"],["&sube;","&#8838;"],["&supe;","&#8839;"],["&oplus;","&#8853;"],["&otimes;","&#8855;"],["&perp;","&#8869;"],["&sdot;","&#8901;"],["&lceil;","&#8968;"],["&rceil;","&#8969;"],["&lfloor;","&#8970;"],["&rfloor;","&#8971;"],["&lang;","&#9001;"],["&rang;","&#9002;"],["&loz;","&#9674;"],["&spades;","&#9824;"],["&clubs;","&#9827;"],["&hearts;","&#9829;"],["&diams;","&#9830;"]]};MooEditable.UI.CharacterDialog=function(d){var c='character <select class="char">';var e=MooEditable.Actions.Settings.charmap.chars;for(var b=0,a=e.length;b<a;b++){c+='<option data-code="'+e[b][0]+'">'+e[b][1]+"</option>"}c+='</select><button class="dialog-button dialog-ok-button">OK</button><button class="dialog-button dialog-cancel-button">Cancel</button>';return new MooEditable.UI.Dialog(c,{"class":"mooeditable-prompt-dialog",onClick:function(h){if(h.target.tagName.toLowerCase()=="button"){h.preventDefault()}var f=document.id(h.target);if(f.hasClass("dialog-cancel-button")){this.close()}else{if(f.hasClass("dialog-ok-button")){this.close();var g=f.getPrevious("select.char");var j=new Element("div").set("html",g.options[g.selectedIndex].getProperty("data-code").trim());d.selection.insertContent(j.get("html"))}}}})};MooEditable.Actions.extend({charmap:{title:"Insert custom character",dialogs:{prompt:function(a){return MooEditable.UI.CharacterDialog(a)}},command:function(){this.dialogs.charmap.prompt.open()},events:{toggleView:function(){if(this.mode=="textarea"){var a=this.textarea.get("value");MooEditable.Actions.Settings.charmap.chars.each(function(c){if(!["&amp;","&gt;","&lt;","&quot;","&nbsp;"].contains(c[0])){var b=new RegExp(String.fromCharCode(parseInt(c[1].replace("&#","").replace(";",""))),"g");a=a.replace(b,c[0])}},this);this.textarea.set("value",a)}}}}});MooEditable.UI.ButtonOverlay=new Class({Extends:MooEditable.UI.Button,options:{overlayHTML:"",overlayClass:"",overlaySize:{x:150,y:"auto"},overlayContentClass:""},initialize:function(a){this.parent(a);this.render();this.el.addClass("mooeditable-ui-buttonOverlay");this.renderOverlay(this.options.overlayHTML)},renderOverlay:function(b){var a=this;this.overlay=new Element("div",{"class":"mooeditable-ui-button-overlay "+a.name+"-overlay "+a.options.overlayClass,html:'<div class="overlay-content '+a.options.overlayContentClass+'">'+b+"</div>",tabindex:0,styles:{left:"-999em",position:"absolute",width:a.options.overlaySize.x,height:a.options.overlaySize.y},events:{mousedown:a.clickOverlay.bind(a),focus:a.openOverlay.bind(a),blur:a.closeOverlay.bind(a)}}).inject(document.body).store("MooEditable.UI.ButtonOverlay",this);this.overlayVisible=false},openOverlay:function(){if(this.overlayVisible){return}this.overlayVisible=true;this.activate();this.fireEvent("openOverlay",this);return this},closeOverlay:function(){if(!this.overlayVisible){return}this.overlay.setStyle("left","-999em");this.overlayVisible=false;this.deactivate();this.fireEvent("closeOverlay",this);return this},clickOverlay:function(a){if(a.target==this.overlay||a.target.parentNode==this.overlay){return}this.overlay.blur();a.preventDefault();this.action(a)},click:function(b){b.preventDefault();if(this.disabled){return}if(this.overlayVisible){this.overlay.blur();return}else{var a=this.el.getCoordinates();this.overlay.setStyles({top:a.bottom,left:a.left});this.overlay.focus()}}});MooEditable.Actions.Settings.forecolor={colors:[["000000","993300","333300","003300","003366","000077","333399","333333"],["770000","ff6600","777700","007700","007777","0000ff","666699","777777"],["ff0000","ff9900","99cc00","339966","33cccc","3366f0","770077","999999"],["ff00ff","ffcc00","ffff00","00ff00","00ffff","00ccff","993366","cccccc"],["ff99cc","ffcc99","ffff99","ccffcc","ccffff","99ccff","cc9977","ffffff"]]};MooEditable.Actions.forecolor={type:"button-overlay",title:"Change Color",options:{overlaySize:{x:"auto"},overlayHTML:(function(){var a="";MooEditable.Actions.Settings.forecolor.colors.each(function(b){b.each(function(d){a+='<a href="#" class="forecolor-colorpicker-color" style="background-color: #'+d+'" title="#'+d.toUpperCase()+'"></a>'});a+='<span class="forecolor-colorpicker-br"></span>'});return a})()},command:function(d,c){var b=c.target;if(b.tagName.toLowerCase()!="a"){return}var a=$(b).getStyle("background-color");this.execute("forecolor",false,a);this.focus()}};MooEditable.Actions.extend({formatBlock:{title:"Block Formatting",type:"menu-list",options:{list:[{text:"Paragraph",value:"p"},{text:"Heading 1",value:"h1",style:"font-size:24px; font-weight:bold;"},{text:"Heading 2",value:"h2",style:"font-size:18px; font-weight:bold;"},{text:"Heading 3",value:"h3",style:"font-size:14px; font-weight:bold;"}]},states:{tags:["p","h1","h2","h3"]},command:function(c,a){var b="<"+a+">";this.focus();this.execute("formatBlock",false,b)}},justifyleft:{title:"Align Left",states:{css:{"text-align":"left"}}},justifyright:{title:"Align Right",states:{css:{"text-align":"right"}}},justifycenter:{title:"Align Center",states:{tags:["center"],css:{"text-align":"center"}}},justifyfull:{title:"Align Justify",states:{css:{"text-align":"justify"}}},removeformat:{title:"Remove Formatting"},insertHorizontalRule:{title:"Insert Horizontal Rule",states:{tags:["hr"]},command:function(){this.selection.insertContent("<hr>")}}});MooEditable.Actions.Settings.smiley={imagesPath:"../../Assets/MooEditable/Smiley/",smileys:["angryface","blush","gasp","grin","halo","lipsaresealed","smile","undecided","wink"],fileExt:".png"};MooEditable.Actions.smiley={type:"button-overlay",title:"Insert Smiley",options:{overlaySize:{x:"auto"},overlayHTML:(function(){var b=MooEditable.Actions.Settings.smiley;var a="";b.smileys.each(function(c){a+='<img src="'+b.imagesPath+c+b.fileExt+'" alt="" class="smiley-image">'});return a})()},command:function(f,c){var a=c.target;if(a.tagName.toLowerCase()!="img"){return}var d=$(a).get("src");var b='<img style="border:0;" class="smiley" src="'+d+'" alt="">';this.focus();this.selection.insertContent(b)},events:{attach:function(a){if(Browser.Engine.trident){a.doc.addListener("controlselect",function(c){var b=c.target;if(b.tagName.toLowerCase()!="img"){return}if(!$(b).hasClass("smiley")){return}c.preventDefault()})}},editorMouseDown:function(d,c){var b=d.target;var a=(b.tagName.toLowerCase()=="img")&&$(b).hasClass("smiley");$try(function(){c.doc.execCommand("enableObjectResizing",false,!a)})}}};MooEditable.Group=new Class({Implements:[Options],options:{actions:"bold italic underline strikethrough | insertunorderedlist insertorderedlist indent outdent | undo redo | createlink unlink | urlimage | toggleview"},initialize:function(a,c){this.setOptions(c);this.actions=this.options.actions.clean().split(" ");var b=this;this.toolbar=new MooEditable.UI.Toolbar({onItemAction:function(){var d=$splat(arguments);var e=d[0];if(!b.activeEditor){return}b.activeEditor.focus();b.activeEditor.action(e.name,d);if(b.activeEditor.mode=="iframe"){b.activeEditor.checkStates()}}}).render(this.actions);document.id(a).adopt(this.toolbar)},add:function(a,b){return this.activeEditor=new MooEditable.Group.Item(a,this,$merge({toolbar:false},this.options,b))}});MooEditable.Group.Item=new Class({Extends:MooEditable,initialize:function(b,d,c){this.group=d;this.parent(b,c);var a=function(){this.group.activeEditor=this}.bind(this);this.textarea.addEvent("focus",a);this.win.addEvent("focus",a)}});MooEditable.UI.MenuList=new Class({Implements:[Events,Options],options:{title:"",name:"","class":"",list:[]},initialize:function(a){this.setOptions(a);this.name=this.options.name;this.render()},toElement:function(){return this.el},render:function(){var a=this;var b="";this.options.list.each(function(c){b+='<option value="{value}" style="{style}">{text}</option>'.substitute(c)});this.el=new Element("select",{"class":a.options["class"],title:a.options.title,html:b,styles:{height:"21px"},events:{change:a.change.bind(a)}});this.disabled=false;if(Browser.Engine.trident){this.el.addEvents({mouseenter:function(c){this.addClass("hover")},mouseleave:function(c){this.removeClass("hover")}})}return this},change:function(b){b.preventDefault();if(this.disabled){return}var a=b.target.value;this.action(a)},action:function(){this.fireEvent("action",[this].concat($A(arguments)))},enable:function(){if(!this.disabled){return}this.disabled=false;this.el.set("disabled",false).removeClass("disabled").set({disabled:false,opacity:1});return this},disable:function(){if(this.disabled){return}this.disabled=true;this.el.set("disabled",true).addClass("disabled").set({disabled:true,opacity:0.4});return this},activate:function(b){if(this.disabled){return}var a=0;if(b){this.options.list.each(function(d,c){if(d.value==b){a=c}})}this.el.selectedIndex=a;return this},deactivate:function(){this.el.selectedIndex=0;this.el.removeClass("onActive");return this}});MooEditable.UI.TableDialog=function(c,b){var a={tableadd:'columns <input type="text" class="table-c" value="" size="4" /> rows <input type="text" class="table-r" value="" size="4" /> ',tableedit:'width <input type="text" class="table-w" value="" size="4" /> class <input type="text" class="table-c" value="" size="15" /> ',tablerowedit:'class <input type="text" class="table-c" value="" size="15" /> type <select class="table-c-type"><option value="th">Header</option><option value="td">Cell</option></select> ',tablecoledit:'width <input type="text" class="table-w" value="" size="4" /> class <input type="text" class="table-c" value="" size="15" /> align <select class="table-a"><option>none</option><option>left</option><option>center</option><option>right</option></select> valign <select class="table-va"><option>none</option><option>top</option><option>middle</option><option>bottom</option></select> '};a[b]+='<button class="dialog-button dialog-ok-button">OK</button><button class="dialog-button dialog-cancel-button">Cancel</button>';var d={tableadd:{click:function(m){var h=this.el.getElement(".table-c").value.toInt();var p=this.el.getElement(".table-r").value.toInt();if(!(p>0&&h>0)){return}var g,o,j,l=[];g=new Element("tdiv");o=new Element("table").set("border",0).set("width","100%").inject(g);j=new Element("tbody").inject(o);for(var f=0;f<p;f++){l[f]=new Element("tr").inject(j,"bottom");for(var n=0;n<h;n++){new Element("td").set("html","&nbsp;").inject(l[f],"bottom")}}c.selection.insertContent(g.get("html"))}},tableedit:{load:function(g){var f=c.selection.getNode().getParent("table");this.el.getElement(".table-w").set("value",f.get("width"));this.el.getElement(".table-c").set("value",f.className)},click:function(g){var f=c.selection.getNode().getParent("table");f.set("width",this.el.getElement(".table-w").value);f.className=this.el.getElement(".table-c").value}},tablerowedit:{load:function(g){var f=c.selection.getNode().getParent("tr");this.el.getElement(".table-c").set("value",f.className);this.el.getElement(".table-c-type").set("value",c.selection.getNode().get("tag"))},click:function(g){var f=c.selection.getNode().getParent("tr");f.className=this.el.getElement(".table-c").value;f.getElements("td,th").each(function(h){if(this.el.getElement(".table-c-type")!=h.get("tag")){var e=c.doc.createElement(this.el.getElement(".table-c-type").get("value"));$(e).set("html",h.get("html")).replaces(h)}},this)}},tablecoledit:{load:function(g){var f=c.selection.getNode();if(f.get("tag")!="td"){f=f.getParent("td")}this.el.getElement(".table-w").set("value",f.get("width"));this.el.getElement(".table-c").set("value",f.className);this.el.getElement(".table-a").set("value",f.get("align"));this.el.getElement(".table-va").set("value",f.get("valign"))},click:function(g){var f=c.selection.getNode();if(f.get("tag")!="td"){f=f.getParent("td")}f.set("width",this.el.getElement(".table-w").value);f.className=this.el.getElement(".table-c").value;f.set("align",this.el.getElement(".table-a").value);f.set("valign",this.el.getElement(".table-va").value)}}};return new MooEditable.UI.Dialog(a[b],{"class":"mooeditable-prompt-dialog",onOpen:function(){if(d[b].load){d[b].load.apply(this)}var e=this.el.getElement("input");(function(){e.focus()}).delay(10)},onClick:function(g){if(g.target.tagName.toLowerCase()=="button"){g.preventDefault()}var f=document.id(g.target);if(f.hasClass("dialog-cancel-button")){this.close()}else{if(f.hasClass("dialog-ok-button")){this.close();d[b].click.apply(this)}}}})};MooEditable.Actions.extend({tableadd:{title:"Add Table",dialogs:{prompt:function(a){return MooEditable.UI.TableDialog(a,"tableadd")}},command:function(){this.dialogs.tableadd.prompt.open()}},tableedit:{title:"Edit Table",dialogs:{prompt:function(a){return MooEditable.UI.TableDialog(a,"tableedit")}},command:function(){if(this.selection.getNode().getParent("table")){this.dialogs.tableedit.prompt.open()}}},tablerowadd:{title:"Add Row",command:function(){var a=this.selection.getNode().getParent("tr");if(a){a.clone().inject(a,"after")}}},tablerowedit:{title:"Edit Row",dialogs:{prompt:function(a){return MooEditable.UI.TableDialog(a,"tablerowedit")}},command:function(){if(this.selection.getNode().getParent("table")){this.dialogs.tablerowedit.prompt.open()}}},tablerowspan:{title:"Merge Row",command:function(){var b=this.selection.getNode();if(b.get("tag")!="td"){b=b.getParent("td")}if(b){var a=b.cellIndex;var c=b.getParent().rowIndex;if(b.getParent().getParent().childNodes[c+b.rowSpan]){b.getParent().getParent().childNodes[c+b.rowSpan].deleteCell(a);b.rowSpan++}}}},tablerowsplit:{title:"Split Row",command:function(){var b=this.selection.getNode();if(b.get("tag")!="td"){b=b.getParent("td")}if(b){var a=b.cellIndex;var d=b.getParent().rowIndex;if(b.getProperty("rowspan")){var c=parseInt(b.getProperty("rowspan"));for(i=1;i<c;i++){b.getParent().getParent().childNodes[d+i].insertCell(a)}b.removeProperty("rowspan")}}},states:function(a){if(a.get("tag")!="td"){return}if(a){if(a.getProperty("rowspan")&&parseInt(a.getProperty("rowspan"))>1){this.el.addClass("onActive")}}}},tablerowdelete:{title:"Delete Row",command:function(){var a=this.selection.getNode().getParent("tr");if(a){a.getParent().deleteRow(a.rowIndex)}}},tablecoladd:{title:"Add Column",command:function(){var e=this.selection.getNode();if(e.get("tag")!="td"){e=e.getParent("td")}if(e){var b=e.cellIndex;var a=e.getParent().getParent().childNodes.length;for(var c=0;c<a;c++){var d=$(e.getParent().getParent().childNodes[c].childNodes[b]);d.clone().inject(d,"after")}}}},tablecoledit:{title:"Edit Column",dialogs:{prompt:function(a){return MooEditable.UI.TableDialog(a,"tablecoledit")}},command:function(){if(this.selection.getNode().getParent("table")){this.dialogs.tablecoledit.prompt.open()}}},tablecolspan:{title:"Merge Cell",command:function(){var b=this.selection.getNode();if(b.get("tag")!="td"){b=b.getParent("td")}if(b){var a=b.cellIndex+1;if(b.getParent().childNodes[a]){b.getParent().deleteCell(a);b.colSpan++}}}},tablecolsplit:{title:"Split Cell",command:function(){var b=this.selection.getNode();if(b.get("tag")!="td"){b=b.getParent("td")}if(b){var a=b.cellIndex+1;if(b.getProperty("colspan")){var c=parseInt(b.getProperty("colspan"));for(i=1;i<c;i++){b.getParent().insertCell(a+i)}b.removeProperty("colspan")}}},states:function(a){if(a.get("tag")!="td"){return}if(a){if(a.getProperty("colspan")&&parseInt(a.getProperty("colspan"))>1){this.el.addClass("onActive")}}}},tablecoldelete:{title:"Delete Column",command:function(){var e=this.selection.getNode();if(e.get("tag")!="td"){e=e.getParent("td")}if(e){var a=e.getParent().getParent().childNodes.length;var b=e.cellIndex;var d=e.getParent().getParent();for(var c=0;c<a;c++){d.childNodes[c].deleteCell(b)}}}}});MooEditable.Actions.Settings.pagebreak={imageFile:"../../Assets/MooEditable/Other/pagebreak.gif"};MooEditable.Actions.extend({pagebreak:{title:"Page break",command:function(){this.selection.insertContent('<img class="mooeditable-visual-aid mooeditable-pagebreak" />')},events:{beforeToggleView:function(){if(this.mode=="iframe"){var a=this.getContent().replace(/<img([^>]*)class="mooeditable-visual-aid mooeditable-pagebreak"([^>]*)>/gi,"<!-- page break -->");this.setContent(a)}else{var a=this.textarea.get("value").replace(/<!-- page break -->/gi,'<img class="mooeditable-visual-aid mooeditable-pagebreak" />');this.textarea.set("value",a)}},render:function(){this.options.extraCSS="img.mooeditable-pagebreak { display:block; width:100%; height:16px; background: url("+MooEditable.Actions.Settings.pagebreak.imageFile+") repeat-x; }"+this.options.extraCSS}}}});MooEditable.UI.FlashDialog=function(b){var a='embed <textarea class="dialog-f" value="" rows="2" cols="40" /></textarea> <button class="dialog-button dialog-ok-button">OK</button> <button class="dialog-button dialog-cancel-button">Cancel</button>';return new MooEditable.UI.Dialog(a,{"class":"mooeditable-prompt-dialog",onOpen:function(){var c=this.el.getElement(".dialog-f");(function(){c.focus();c.select()}).delay(10)},onClick:function(d){if(d.target.tagName.toLowerCase()=="button"){d.preventDefault()}var c=document.id(d.target);if(c.hasClass("dialog-cancel-button")){this.close()}else{if(c.hasClass("dialog-ok-button")){this.close();var f=new Element("div").set("html",this.el.getElement(".dialog-f").get("value").trim());b.selection.insertContent(f.get("html"))}}}})};MooEditable.Actions.extend({flash:{title:"Flash embed",dialogs:{prompt:function(a){return MooEditable.UI.FlashDialog(a)}},command:function(){this.dialogs.flash.prompt.open()}}});MooEditable.UI.ImageDialog=function(b){var a='url <input type="text" class="dialog-url" value="" size="15" /> alt <input type="text" class="dialog-alt" value="" size="8" /> class <input type="text" class="dialog-class" value="" size="8" /> align <select class="dialog-align"><option>none</option><option>left</option><option>center</option><option>right</option></select> <button class="dialog-button dialog-ok-button">OK</button> <button class="dialog-button dialog-cancel-button">Cancel</button>';return new MooEditable.UI.Dialog(a,{"class":"mooeditable-prompt-dialog",onOpen:function(){var c=this.el.getElement(".dialog-url");var d=b.selection.getNode();if(d.get("tag")=="img"){this.el.getElement(".dialog-url").set("value",d.get("src"));this.el.getElement(".dialog-alt").set("value",d.get("alt"));this.el.getElement(".dialog-class").set("value",d.className);this.el.getElement(".dialog-align").set("align",d.get("align"))}(function(){c.focus();c.select()}).delay(10)},onClick:function(f){if(f.target.tagName.toLowerCase()=="button"){f.preventDefault()}var c=document.id(f.target);if(c.hasClass("dialog-cancel-button")){this.close()}else{if(c.hasClass("dialog-ok-button")){this.close();var d=b.selection.getNode();if(d.get("tag")=="img"){d.set("src",this.el.getElement(".dialog-url").get("value").trim());d.set("alt",this.el.getElement(".dialog-alt").get("value").trim());d.className=this.el.getElement(".dialog-class").get("value").trim();d.set("align",this.el.getElement(".dialog-align").get("value"))}else{var g=new Element("div");new Element("img",{src:this.el.getElement(".dialog-url").get("value").trim(),alt:this.el.getElement(".dialog-alt").get("value").trim(),"class":this.el.getElement(".dialog-class").get("value").trim(),align:this.el.getElement(".dialog-align").get("value")}).inject(g);b.selection.insertContent(g.get("html"))}}}}})};MooEditable.Actions.extend({image:{title:"Add/Edit Image",options:{shortcut:"m"},dialogs:{prompt:function(a){return MooEditable.UI.ImageDialog(a)}},command:function(){this.dialogs.image.prompt.open()}}});
